@@ -38,7 +38,7 @@ class ArangoClient(object):
                  port=8529,
                  username='root',
                  password='',
-                 verify=True,
+                 verify=False,
                  http_client=None,
                  enable_logging=True):
 
@@ -61,15 +61,25 @@ class ArangoClient(object):
         )
         self._wal = WriteAheadLog(self._conn)
 
-        # Verify the server connection
         if verify:
-            res = self._conn.head('/_api/version')
-            if res.status_code not in HTTP_OK:
-                raise ServerConnectionError(res)
+            self.verify()
 
     def __repr__(self):
         return '<ArangoDB client for "{}">'.format(self._host)
 
+    def verify(self):
+        """Verify the connection to ArangoDB server.
+
+        :returns: ``True`` if the connection is successful
+        :rtype: bool
+        :raises arango.exceptions.ServerConnectionError: if the connection to
+            the ArangoDB server fails
+        """
+        res = self._conn.head('/_api/version')
+        if res.status_code not in HTTP_OK:
+            raise ServerConnectionError(res)
+        return True
+
     @property
     def protocol(self):
         """Return the internet transfer protocol.
@@ -147,7 +157,7 @@ class ArangoClient(object):
 
         :returns: the server version
         :rtype: str
-        :raises arango.exceptions.ServerGetVersionError: if the server version
+        :raises arango.exceptions.ServerVersionError: if the server version
             cannot be retrieved
         """
         res = self._conn.get(
@@ -155,7 +165,7 @@ class ArangoClient(object):
             params={'details': False}
         )
         if res.status_code not in HTTP_OK:
-            raise ServerGetVersionError(res)
+            raise ServerVersionError(res)
         return res.body['version']
 
     def details(self):
@@ -163,7 +173,7 @@ class ArangoClient(object):
 
         :returns: the server details
         :rtype: dict
-        :raises arango.exceptions.ServerGetDetailsError: if the server details
+        :raises arango.exceptions.ServerDetailsError: if the server details
             cannot be retrieved
         """
         res = self._conn.get(
@@ -171,7 +181,7 @@ class ArangoClient(object):
             params={'details': True}
         )
         if res.status_code not in HTTP_OK:
-            raise ServerGetDetailsError(res)
+            raise ServerDetailsError(res)
         return res.body['details']
 
     def required_db_version(self):
@@ -179,12 +189,12 @@ class ArangoClient(object):
 
         :returns: the required version of the target database
         :rtype: str
-        :raises arango.exceptions.ServerGetRequiredVersionError: if the
+        :raises arango.exceptions.ServerRequiredDBVersionError: if the
             required database version cannot be retrieved
         """
         res = self._conn.get('/_admin/database/target-version')
         if res.status_code not in HTTP_OK:
-            raise ServerGetRequiredVersionError(res)
+            raise ServerRequiredDBVersionError(res)
         return res.body['version']
 
     def statistics(self, description=False):
@@ -192,7 +202,7 @@ class ArangoClient(object):
 
         :returns: the statistics information
         :rtype: dict
-        :raises arango.exceptions.ServerGetStatisticsError: if the server
+        :raises arango.exceptions.ServerStatisticsError: if the server
             statistics cannot be retrieved
         """
         res = self._conn.get(
@@ -200,7 +210,7 @@ class ArangoClient(object):
             if description else '/_admin/statistics'
         )
         if res.status_code not in HTTP_OK:
-            raise ServerGetStatisticsError(res)
+            raise ServerStatisticsError(res)
         res.body.pop('code', None)
         res.body.pop('error', None)
         return res.body
@@ -215,12 +225,12 @@ class ArangoClient(object):
             in the cluster) or ``"UNDEFINED"`` (the server role is undefined,
             the only possible value for a single server)
         :rtype: str
-        :raises arango.exceptions.ServerGetRoleError: if the server role cannot
+        :raises arango.exceptions.ServerRoleError: if the server role cannot
             be retrieved
         """
         res = self._conn.get('/_admin/server/role')
         if res.status_code not in HTTP_OK:
-            raise ServerGetRoleError(res)
+            raise ServerRoleError(res)
         return res.body.get('role')
 
     def time(self):
@@ -228,12 +238,12 @@ class ArangoClient(object):
 
         :returns: the server system time
         :rtype: datetime.datetime
-        :raises arango.exceptions.ServerGetTimeError: if the server time
+        :raises arango.exceptions.ServerTimeError: if the server time
             cannot be retrieved
         """
         res = self._conn.get('/_admin/time')
         if res.status_code not in HTTP_OK:
-            raise ServerGetTimeError(res)
+            raise ServerTimeError(res)
         return datetime.fromtimestamp(res.body['time'])
 
     def endpoints(self):
@@ -246,18 +256,18 @@ class ArangoClient(object):
 
         :returns: the list of endpoints
         :rtype: list
-        :raises arango.exceptions.ServerGetEndpointsError: if the endpoints
+        :raises arango.exceptions.ServerEndpointsError: if the endpoints
             cannot be retrieved from the server
         """
         res = self._conn.get('/_api/endpoint')
         if res.status_code not in HTTP_OK:
-            raise ServerGetEndpointsError(res)
+            raise ServerEndpointsError(res)
         return res.body
 
     def echo(self):
         """Return information on the last request (headers, payload etc.)
 
-        :returns: the information on the last request
+        :returns: the details of the last request
         :rtype: dict
         :raises arango.exceptions.ServerEchoError: if the last request cannot
             be retrieved from the server
@@ -285,7 +295,7 @@ class ArangoClient(object):
             raise ServerSleepError(res)
         return res.body['duration']
 
-    def shutdown(self):
+    def shutdown(self):  # pragma: no cover
         """Initiate the server shutdown sequence.
 
         :returns: whether the server was shutdown successfully
@@ -301,7 +311,7 @@ class ArangoClient(object):
             raise ServerShutdownError(res)
         return True
 
-    def run_tests(self, tests):
+    def run_tests(self, tests):  # pragma: no cover
         """Run the available unittests on the server.
 
         :param tests: list of files containing the test suites
@@ -384,7 +394,7 @@ class ArangoClient(object):
             params['sort'] = sort
         res = self._conn.get('/_admin/log')
         if res.status_code not in HTTP_OK:
-            ServerReadLogError(res)
+            raise ServerReadLogError(res)
         if 'totalAmount' in res.body:
             res.body['total_amount'] = res.body.pop('totalAmount')
         return res.body
@@ -413,8 +423,7 @@ class ArangoClient(object):
         :type user_only: bool
         :returns: the database names
         :rtype: list
-        :raises arango.exceptions.DatabasesListError: if the database names
-            cannot be retrieved from the server
+        :raises arango.exceptions.DatabaseListError: if the retrieval fails
         """
         # Get the current user's databases
         res = self._conn.get(
@@ -422,7 +431,7 @@ class ArangoClient(object):
             if user_only else '/_api/database'
         )
         if res.status_code not in HTTP_OK:
-            raise DatabasesListError(res)
+            raise DatabaseListError(res)
         return res.body['result']
 
     def db(self, name, username=None, password=None):
@@ -432,11 +441,11 @@ class ArangoClient(object):
 
         :param name: the name of the database
         :type name: str
-        :param username: the username for the database connection (if set,
-            overrides the username specified during the client initialization)
+        :param username: the username for authentication (if set, overrides
+            the username specified during the client initialization)
         :type username: str
-        :param password: the password for the database connection (if set,
-            overrides the password specified during the client initialization
+        :param password: the password for authentication (if set, overrides
+            the password specified during the client initialization
         :type password: str
         :returns: the database object
         :rtype: arango.database.Database
@@ -448,11 +457,11 @@ class ArangoClient(object):
 
         :param name: the name of the database
         :type name: str
-        :param username: the username for the database connection (if set,
-            overrides the username specified during the client initialization)
+        :param username: the username for authentication (if set, overrides
+            the username specified during the client initialization)
         :type username: str
-        :param password: the password for the database connection (if set,
-            overrides the password specified during the client initialization
+        :param password: the password for authentication (if set, overrides
+            the password specified during the client initialization
         :type password: str
         :returns: the database object
         :rtype: arango.database.Database
@@ -468,33 +477,55 @@ class ArangoClient(object):
             enable_logging=self._logging
         ))
 
-    def create_database(self, name, users=None):
+    def create_database(self, name, users=None, username=None, password=None):
         """Create a new database.
 
         :param name: the name of the new database
         :type name: str
-        :param users: the list of database users, where each user must be a
-            sub-dictionary with fields ``"username"``, ``"password"`` and
-            ``"active"`` (if set to ``False`` the user will not be able to
-            log into the database)
-        :type users: list
+        :param users: the list of users with access to the new database, where
+            each user is a dictionary with keys ``"username"``, ``"password"``,
+            ``"active"`` and ``"extra"``.
+        :type users: [dict]
+        :param username: the username for authentication (if set, overrides
+            the username specified during the client initialization)
+        :type username: str
+        :param password: the password for authentication (if set, overrides
+            the password specified during the client initialization
+        :type password: str
         :returns: the database object
         :rtype: arango.database.Database
-        :raises arango.exceptions.DatabaseCreateError: if the database cannot
-            be created on the server
+        :raises arango.exceptions.DatabaseCreateError: if the create fails
 
         .. note::
-            If **users** is not set, only the ``root`` user will have access
-            to the new database by default.
+            Here is an example entry in **users**:
+
+            .. code-block:: python
+
+                {
+                    'username': 'john',
+                    'password': 'password',
+                    'active': True,
+                    'extra': {'Department': 'IT'}
+                }
+
+            If **users** is not set, only the root and the current user are
+            granted access to the new database by default.
         """
         res = self._conn.post(
             '/_api/database',
-            data={'name': name, 'users': users}
-            if users else {'name': name}
+            data={
+                'name': name,
+                'users': [{
+                    'username': user['username'],
+                    'passwd': user['password'],
+                    'active': user.get('active', True),
+                    'extra': user.get('extra', {})
+                } for user in users]
+            } if users else {'name': name}
         )
         if res.status_code not in HTTP_OK:
             raise DatabaseCreateError(res)
-        return self.db(name)
+        return self.db(name, username, password)
 
     def delete_database(self, name, ignore_missing=False):
         """Delete the database of the specified name.
@@ -505,8 +536,7 @@ class ArangoClient(object):
         :type ignore_missing: bool
         :returns: whether the database was deleted successfully
         :rtype: bool
-        :raises arango.exceptions.DatabaseDeleteError: if the database cannot
-            be deleted from the server
+        :raises arango.exceptions.DatabaseDeleteError: if the delete fails
         """
         res = self._conn.delete('/_api/database/{}'.format(name))
         if res.status_code not in HTTP_OK:
@@ -519,33 +549,37 @@ class ArangoClient(object):
     ###################
 
     def users(self):
-        """Return the details on all users.
+        """Return the details of all users.
 
-        :returns: the mapping of usernames to user details
-        :rtype: list
-        :raises arango.exceptions.UsersListError: if the details on the users
-            cannot be retrieved from the server
+        :returns: the details of all users
+        :rtype: [dict]
+        :raises arango.exceptions.UserListError: if the retrieval fails
+
+        .. note::
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
         res = self._conn.get('/_api/user')
         if res.status_code not in HTTP_OK:
-            raise UsersListError(res)
-
+            raise UserListError(res)
         return [{
             'username': record['user'],
             'active': record['active'],
             'extra': record['extra'],
-            'change_password': record['changePassword']
-        } for record in map(dict, res.body['result'])]
+        } for record in res.body['result']]
 
     def user(self, username):
-        """Return the details on a user
+        """Return the details of a user.
 
-        :param username: the details on the user
+        :param username: the details of the user
         :type username: str
         :returns: the user details
         :rtype: dict
-        :raises arango.exceptions.UserGetError: if the details on user cannot
-            be retrieved from the server
+        :raises arango.exceptions.UserGetError: if the retrieval fails
+
+        .. note::
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
         res = self._conn.get('/_api/user/{}'.format(username))
         if res.status_code not in HTTP_OK:
@@ -553,44 +587,33 @@ class ArangoClient(object):
         return {
             'username': res.body['user'],
             'active': res.body['active'],
-            'extra': res.body['extra'],
-            'change_password': res.body['changePassword']
+            'extra': res.body['extra']
         }
 
-    def create_user(self,
-                    username,
-                    password,
-                    active=None,
-                    extra=None,
-                    change_password=None):
+    def create_user(self, username, password, active=None, extra=None):
         """Create a new user.
 
         :param username: the name of the user
         :type username: str
-        :param password: the user password
+        :param password: the user's password
         :type password: str
         :param active: whether the user is active
         :type active: bool
-        :param extra: any extra data about the user
+        :param extra: any extra data on the user
         :type extra: dict
-        :param change_password: if ``True``, the specified user will be forced
-            to change his/her password and the only operation allowed by the
-            user will be :func:`~arango.client.ArangoClient.replace_user` or
-            :func:`~arango.client.ArangoClient.update_user` (all other calls
-            will result in an HTTP 403)
-        :type change_password: bool
-        :returns: the information about the new user
+        :returns: the details of the new user
         :rtype: dict
-        :raises arango.exceptions.UserCreateError: if the new user cannot be
-            created on the server
+        :raises arango.exceptions.UserCreateError: if the user create fails
+
+        .. note::
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
         data = {'user': username, 'passwd': password}
         if active is not None:
             data['active'] = active
         if extra is not None:
             data['extra'] = extra
-        if change_password is not None:
-            data['changePassword'] = change_password
 
         res = self._conn.post('/_api/user', data=data)
         if res.status_code not in HTTP_OK:
@@ -599,48 +622,38 @@ class ArangoClient(object):
             'username': res.body['user'],
             'active': res.body['active'],
             'extra': res.body['extra'],
-            'change_password': res.body['changePassword'],
         }
 
-    def update_user(self,
-                    username,
-                    password=None,
-                    active=None,
-                    extra=None,
-                    change_password=None):
+    def update_user(self, username, password=None, active=None, extra=None):
         """Update an existing user.
 
         :param username: the name of the existing user
         :type username: str
-        :param password: the user password
+        :param password: the user's new password
         :type password: str
         :param active: whether the user is active
         :type active: bool
-        :param extra: any extra data about the user
+        :param extra: any extra data on the user
         :type extra: dict
-        :param change_password: if ``True``, the specified user will be forced
-            to change his/her password and the only operation allowed by the
-            user will be :func:`~arango.client.ArangoClient.replace_user` or
-            :func:`~arango.client.ArangoClient.update_user` (all other calls
-            will result in an HTTP 403)
-        :type change_password: bool
-        :returns: the information about the updated user
+        :returns: the details of the updated user
         :rtype: dict
-        :raises arango.exceptions.UserUpdateError: if the specified user
-            cannot be updated on the server
+        :raises arango.exceptions.UserUpdateError: if the user update fails
+
+        .. note::
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
         data = {}
         if password is not None:
-            data['password'] = password
+            data['passwd'] = password
         if active is not None:
             data['active'] = active
         if extra is not None:
             data['extra'] = extra
-        if change_password is not None:
-            data['changePassword'] = change_password
 
         res = self._conn.patch(
-            '/_api/user/{user}'.format(user=username), data=data
+            '/_api/user/{user}'.format(user=username),
+            data=data
         )
         if res.status_code not in HTTP_OK:
             raise UserUpdateError(res)
@@ -648,50 +661,36 @@ class ArangoClient(object):
             'username': res.body['user'],
             'active': res.body['active'],
             'extra': res.body['extra'],
-            'change_password': res.body['changePassword'],
         }
 
-    def replace_user(self,
-                     username,
-                     password,
-                     active=None,
-                     extra=None,
-                     change_password=None):
+    def replace_user(self, username, password, active=None, extra=None):
         """Replace an existing user.
 
-        if ``change_password`` is set to true, the only operation allowed by
-        the user will be ``self.replace_user`` or ``self.update_user``. All
-        other operations executed by the user will result in an HTTP 403.
-
         :param username: the name of the existing user
         :type username: str
-        :param password: the user password
+        :param password: the user's new password
         :type password: str
         :param active: whether the user is active
         :type active: bool
-        :param extra: any extra data about the user
+        :param extra: any extra data on the user
         :type extra: dict
-        :param change_password: if ``True``, the specified user will be forced
-            to change his/her password and the only operation allowed by the
-            user will be :func:`~arango.client.ArangoClient.replace_user` or
-            :func:`~arango.client.ArangoClient.update_user` (all other calls
-            will result in an HTTP 403)
-        :type change_password: bool
-        :returns: the information about the replaced user
+        :returns: the details of the replaced user
         :rtype: dict
-        :raises arango.exceptions.UserReplaceError: if the specified user
-            cannot be replaced on the server
+        :raises arango.exceptions.UserReplaceError: if the user replace fails
+
+        .. note::
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
-        data = {'user': username, 'password': password}
+        data = {'user': username, 'passwd': password}
         if active is not None:
             data['active'] = active
         if extra is not None:
             data['extra'] = extra
-        if change_password is not None:
-            data['changePassword'] = change_password
 
         res = self._conn.put(
-            '/_api/user/{user}'.format(user=username), data=data
+            '/_api/user/{user}'.format(user=username), 
+            data=data
         )
         if res.status_code not in HTTP_OK:
             raise UserReplaceError(res)
@@ -699,27 +698,51 @@ class ArangoClient(object):
             'username': res.body['user'],
             'active': res.body['active'],
             'extra': res.body['extra'],
-            'change_password': res.body['changePassword'],
         }
 
     def delete_user(self, username, ignore_missing=False):
         """Delete an existing user.
 
-        :param username: the name of the user
+        :param username: the name of the existing user
         :type username: str
         :param ignore_missing: ignore missing users
         :type ignore_missing: bool
-        :raises arango.exceptions.UserDeleteError: if the specified user
-            cannot be deleted from the server
+        :returns: ``True`` if the operation was successful, ``False`` if the
+            user was missing but **ignore_missing** was set to ``True``
+        :rtype: bool
+        :raises arango.exceptions.UserDeleteError: if the user delete fails
+
+        .. note::
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
         res = self._conn.delete('/_api/user/{user}'.format(user=username))
-        if res.status_code not in HTTP_OK:
-            if not (res.status_code == 404 and ignore_missing):
-                raise UserDeleteError(res)
-        return not res.body['error']
+        if res.status_code in HTTP_OK:
+            return True
+        elif res.status_code == 404 and ignore_missing:
+            return False
+        raise UserDeleteError(res)
+
+    def user_access(self, username):
+        """Return the database access details of a user.
+
+        :param username: the name of the user
+        :type username: str
+        :returns: the names of the databases the user can access
+        :rtype: [str]
+        :raises: arango.exceptions.UserAccessError: if the retrieval fails
+
+        .. note::
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
+        """
+        res = self._conn.get('/_api/user/{}/database'.format(username))
+        if res.status_code in HTTP_OK:
+            return list(res.body['result'])
+        raise UserAccessError(res)
 
     def grant_user_access(self, username, database):
-        """Grant user access to the given database.
+        """Grant user access to a database.
 
         :param username: the name of the user
         :type username: str
@@ -727,22 +750,22 @@ class ArangoClient(object):
         :type database: str
         :returns: whether the operation was successful
         :rtype: bool
-        :raises arango.exceptions.UserGrantAccessError: if the access cannot
-            be granted to the specified user
+        :raises arango.exceptions.UserGrantAccessError: if the operation fails
 
         .. note::
-            Admin privileges are required to use this method
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
         res = self._conn.put(
             '/_api/user/{}/database/{}'.format(username, database),
             data={'grant': 'rw'}
         )
-        if res.status_code not in HTTP_OK:
-            raise UserGrantAccessError(res)
-        return not res.body.get('error')
+        if res.status_code in HTTP_OK:
+            return True
+        raise UserGrantAccessError(res)
 
     def revoke_user_access(self, username, database):
-        """Revoke user access to the given database.
+        """Revoke user access to a database.
 
         :param username: the name of the user
         :type username: str
@@ -750,16 +773,64 @@ class ArangoClient(object):
         :type database: str
         :returns: whether the operation was successful
         :rtype: bool
-        :raises arango.exceptions.UserRevokeAccessError: if the access cannot
-            be revoked from the specified user
+        :raises arango.exceptions.UserRevokeAccessError: if the operation fails
 
         .. note::
-            Admin privileges are required to use this method
+            Root privileges (i.e. access to the ``_system`` database) are
+            required to use this method
         """
         res = self._conn.put(
             '/_api/user/{}/database/{}'.format(username, database),
             data={'grant': 'none'}
         )
+        if res.status_code in HTTP_OK:
+            return True
+        raise UserRevokeAccessError(res)
+
+    ########################
+    # Async Job Management #
+    ########################
+
+    def async_jobs(self, status, count=None):
+        """Return the IDs of asynchronous jobs with the specified status.
+
+        :param status: the job status (``"pending"`` or ``"done"``)
+        :type status: str
+        :param count: the maximum number of job IDs to return
+        :type count: int
+        :returns: the list of job IDs
+        :rtype: [str]
+        :raises arango.exceptions.AsyncJobListError: if the retrieval fails
+        """
+        res = self._conn.get(
+            '/_api/job/{}'.format(status),
+            params={} if count is None else {'count': count}
+        )
         if res.status_code not in HTTP_OK:
-            raise UserRevokeAccessError(res)
-        return not res.body.get('error')
+            raise AsyncJobListError(res)
+        return res.body
+
+    def clear_async_jobs(self, threshold=None):
+        """Delete asynchronous job results from the server.
+
+        :param threshold: if specified, only the job results created prior to
+            the threshold (a unix timestamp) are deleted, otherwise *all* job
+            results are deleted
+        :type threshold: int
+        :returns: whether the deletion of results was successful
+        :rtype: bool
+        :raises arango.exceptions.AsyncJobClearError: if the operation fails
+
+        .. note::
+            Async jobs currently queued or running are not stopped.
+        """
+        if threshold is None:
+            res = self._conn.delete('/_api/job/all')
+        else:
+            res = self._conn.delete(
+                '/_api/job/expired',
+                params={'stamp': threshold}
+            )
+        if res.status_code in HTTP_OK:
+            return True
+        raise AsyncJobClearError(res)
